//---------------------------------------------------------------------------
//		Header file para acceder a las funciones UDP de la librara de ethernet LwIP
//---------------------------------------------------------------------------

#ifndef Eth_lwipH
#define Eth_lwipH  


#ifdef Eth_NETIF
//----------------------------------------------------------------------------
#include "lwip/err.h"
#include "lwip/ip_addr.h"
#include "lwip/inet.h"
#include "lwip/pbuf.h"
#include "lwip/dhcp.h"
// The list of network interfaces. 
extern struct netif *netif_list;
// The default network interface. 
extern struct netif *netif_default;

// netif_init() must be called first. 
extern void netif_init(void);

extern struct netif *netif_add(struct ip_addr *ipaddr, struct ip_addr *netmask,
      struct ip_addr *gw,
      void *state,
      err_t (* init)(struct netif *netif),
      err_t (* input)(struct pbuf *p, struct netif *netif));

extern void
netif_set_addr(struct netif *netif,struct ip_addr *ipaddr, struct ip_addr *netmask,
    struct ip_addr *gw);
void netif_remove(struct netif * netif);

// Returns a network interface given its name. The name is of the form
//   "et0", where the first two letters are the "name" field in the
//   netif structure, and the digit is in the num field in the same
//   structure. 
extern struct netif *netif_find(char *name);

extern void netif_set_default(struct netif *netif);

extern void netif_set_ipaddr(struct netif *netif, struct ip_addr *ipaddr);
extern void netif_set_netmask(struct netif *netif, struct ip_addr *netmast);
extern void netif_set_gw(struct netif *netif, struct ip_addr *gw);
//----------------------------------------------------------------------------

#endif


#ifdef Eth_ARP			//Se utilizan las funciones externas ARP
#include "netif/arp.h"  //para estructuras eth_addr
#include "lwip/ip_addr.h" //para estructuras ip_addr
#include "lwip/pbuf.h"	//para estructuras pbuf
#include "lwip/netif.h" //para estructuras netif
//-------------------extern  funtions arp.c------------------------
// Initializes ARP. 
extern void arp_init(void);
// Should be called for all incoming packets of IP kind. The function
//   does not alter the packet in any way, it just updates the ARP
//   table. After this function has been called, the normal TCP/IP stack
//   input function should be called. 
extern void arp_ip_input(struct netif *netif, struct pbuf *p);
// Should be called for incoming ARP packets. The pbuf in the argument
//   is freed by this function. If the function returns a pbuf (i.e.,
//   returns non-NULL), that pbuf constitutes an ARP reply and should be
//   sent out on the Ethernet. 
extern struct pbuf *arp_arp_input(struct netif *netif, struct eth_addr *ethaddr,
			   struct pbuf *p);
// arp_loopup() is called to do an IP address -> Ethernet address
//   translation. If the function returns NULL, there is no mapping and
//   the arp_query() function should be called. 
extern struct eth_addr *arp_lookup(struct ip_addr *ipaddr);

// Constructs an ARP query packet for the given IP address. The
// function returns a pbuf that contains the reply and that should be
// sent out on the Ethernet. 
extern struct pbuf *arp_query(struct netif *netif,  struct eth_addr *ethaddr,
		        struct ip_addr *ipaddr);
//----------------------------------------------------------------------------

#endif

#ifdef Eth_ETHARP
#include "lwip/pbuf.h"
#include "lwip/ip_addr.h"
#include "lwip/netif.h"
#include "lwip/ip.h"
//----------------------------------------------------------------------------

extern void etharp_init(void);
extern void etharp_tmr(void);
extern struct pbuf *etharp_ip_input(struct netif *netif, struct pbuf *p);
extern struct pbuf *etharp_arp_input(struct netif *netif, struct eth_addr *ethaddr,
         struct pbuf *p);
extern struct pbuf *etharp_output(struct netif *netif, struct ip_addr *ipaddr,
         struct pbuf *q);
extern err_t etharp_query(struct netif *netif, struct ip_addr *ipaddr, struct pbuf *q);
//----------------------------------------------------------------------------

#endif

#ifdef   Eth_UDP
#include "lwip/udp.h"  
#include "lwip/pbuf.h"	//para estructuras pbuf
#include "lwip/ip_addr.h" //para estructuras ip_addr
#include "lwip/ip.h"	//para estructuras ip_hdr

/* The following functions is the application layer interface to the
   UDP code. */
//---------------------------------------------------------------------------
extern struct udp_pcb * 	udp_new        (void);
extern void             	udp_remove     (struct udp_pcb *pcb);
extern err_t            	udp_bind       (struct udp_pcb *pcb, struct ip_addr *ipaddr, u16_t port);
extern err_t            	udp_connect    (struct udp_pcb *pcb, struct ip_addr *ipaddr,u16_t port);
extern void             	udp_disconnect (struct udp_pcb *pcb);
extern void             	udp_recv       (struct udp_pcb *pcb, void (* recv)(void *arg, struct udp_pcb *upcb,struct pbuf *p,struct ip_addr *addr,u16_t port),void *recv_arg);
extern err_t            	udp_send       (struct udp_pcb *pcb, struct pbuf *p);


/* The following functions are the lower layer interface to UDP. */
extern u8_t             udp_lookup     (struct ip_hdr *iphdr, struct netif *inp);
extern void             udp_input      (struct pbuf *p, struct netif *inp);
extern void             udp_init       (void);
extern err_t            udp_bind       (struct udp_pcb *pcb, struct ip_addr *ipaddr,
                 u16_t port);
//extern          udp_flags(pcb)  ((pcb)->flags)
//extern          udp_setflags(pcb, f)  ((pcb)->flags = (f))
//---------------------------------------------------------------------------
#endif
//---------------------------------------------------------------------------

#ifdef Eth_MEM
#include "lwip/mem.h"
//-----------------extern functions for mem module
extern void mem_init(void);

extern void *mem_malloc(mem_size_t size);
extern void mem_free(void *mem);
extern void *mem_realloc(void *mem, mem_size_t size);
extern void *mem_reallocm(void *mem, mem_size_t size);
//---------------------------------------------------------------------------
#endif
//---------------------------------------------------------------------------
#ifdef Eth_MEMP
//-------------------------------------------------------------------------------------

#include "lwip/memp.h"
extern void memp_init(void);

extern void *memp_malloc(memp_t type);
extern void *memp_realloc(memp_t fromtype, memp_t totype, void *mem);
extern void memp_free(memp_t type, void *mem);
extern void *memp_mallocp(memp_t type);
extern void memp_freep(memp_t type, void *mem);
//-------------------------------------------------------------------------------------

#endif

#ifdef Eth_STATS
#include "lwip/stats.h"
#include "lwip/opt.h"	
//---------------------------------------------------------------------------
//-----------------extern functions for stast module
#if LWIP_STATS
	extern void stats_init(void);
	extern struct stats_ lwip_stats;
#else
	extern stats_init()
#endif
//extern void arp_nada(void);
//---------------------------------------------------------------------------
#endif

#ifdef Eth_CS8900IF
#include "cs8900a/cs8900if.h"
#include "lwip/netif.h" //para estructuras netif
//-------------------extern  del device diver cs8900a------------------------
extern void cs8900if_init(struct netif *netif);
extern void cs8900_nada(void);
extern void cs8900_send_debug(unsigned char *p, unsigned int len);
extern void cs8900_init(struct netif *netif);        //Natalia
extern u16_t cs8900_inputData(void);
extern void cs8900_iniVariables(void);
extern void cs8900_nada(void);
//----------------------------------------------------------------------------
#endif

#ifdef Eth_PBUF
#include "lwip/pbuf.h" 

//-----------------Declaracin de funciones mdule pbuf----------------------------------
//   Initializes the pbuf module. The num parameter determines how many
//   pbufs that should be allocated to the pbuf pool, and the size
//   parameter specifies the size of the data allocated to those.  
extern void pbuf_init(void);

extern struct pbuf *pbuf_alloc(pbuf_layer l, u16_t size, pbuf_flag flag);
extern void pbuf_realloc(struct pbuf *p, u16_t size); 
extern u8_t pbuf_header(struct pbuf *p, s16_t header_size);
extern void pbuf_ref(struct pbuf *p);
extern void pbuf_ref_chain(struct pbuf *p);
extern u8_t pbuf_free(struct pbuf *p);
extern u8_t pbuf_clen(struct pbuf *p);  
extern void pbuf_chain(struct pbuf *h, struct pbuf *t);
extern struct pbuf *pbuf_take(struct pbuf *f);
extern struct pbuf *pbuf_dechain(struct pbuf *p);
//-------------------------------------------------------------------------------------
#endif

#ifdef Eth_DHCP
//-------------------------------------------------------------------------------------

#include "lwip/dhcp.h"
#include "lwip/netif.h" //para estructuras netif
#include "lwip/ip_addr.h" //para estructuras ip_addr


/** start DHCP configuration */
extern err_t dhcp_start(struct netif *netif);
/** stop DHCP configuration */
extern void dhcp_stop(struct netif *netif);
/** enforce lease renewal */
extern err_t dhcp_renew(struct netif *netif);
/** inform server of our IP address */
extern void dhcp_inform(struct netif *netif);

/** if enabled, check whether the offered IP address is not in use, using ARP */
#if DHCP_DOES_ARP_CHECK
extern void dhcp_arp_reply(struct netif *netif, struct ip_addr *addr);
#endif

/** to be called every minute */
extern void dhcp_coarse_tmr(void);
/** to be called every half second */
extern void dhcp_fine_tmr(void);
//-------------------------------------------------------------------------------------

#endif



#ifdef Eth_ICMP
#include "lwip/icmp.h"
#include "lwip/pbuf.h"	//para estructuras pbuf
//-------------------------------------------------------------------------------------
extern void icmp_input(struct pbuf *p, struct netif *inp);

extern void icmp_dest_unreach(struct pbuf *p, enum icmp_dur_type t);
extern void icmp_time_exceeded(struct pbuf *p, enum icmp_te_type t);
//-------------------------------------------------------------------------------------
#endif

#ifdef Eth_INET
//-------------------------------------------------------------------------------------

#include "lwip/inet.h"
#include "lwip/pbuf.h"	//para estructuras pbuf
#include "lwip/ip_addr.h" //para estructuras ip_addr, in_addr

extern u16_t inet_chksum(void *dataptr, u16_t len);
extern u16_t inet_chksum_pbuf(struct pbuf *p);
extern u16_t inet_chksum_pseudo(struct pbuf *p,
		 struct ip_addr *src, struct ip_addr *dest,
       u8_t proto, u16_t proto_len);

extern u32_t inet_addr(const char *cp);
extern int inet_aton(const char *cp, struct in_addr *addr);
extern u8_t *inet_ntoa(u32_t addr); /* returns ptr to static buffer; not reentrant! */
//-------------------------------------------------------------------------------------

#endif

#ifdef Eth_IP
//-------------------------------------------------------------------------------------
#include "lwip/arch.h"

#include "lwip/def.h"
#include "lwip/pbuf.h"
#include "lwip/ip_addr.h"

#include "lwip/err.h"
#include "lwip/netif.h"

extern void ip_init(void);
extern u8_t ip_lookup(void *header, struct netif *inp);
extern struct netif *ip_route(struct ip_addr *dest);
extern err_t ip_input(struct pbuf *p, struct netif *inp);
extern err_t ip_output(struct pbuf *p, struct ip_addr *src, struct ip_addr *dest,
    u8_t ttl, u8_t proto);
extern err_t ip_output_if(struct pbuf *p, struct ip_addr *src, struct ip_addr *dest,
       u8_t ttl, u8_t proto,
       struct netif *netif);
//-------------------------------------------------------------------------------------

#endif

#ifdef Eth_IP_ADDR
//-------------------------------------------------------------------------------------
#include "lwip/ip_addr.h"

extern const struct ip_addr ip_addr_any;
extern const struct ip_addr ip_addr_broadcast;

extern void IP4_ADDR(struct ip_addr* ipaddr, u32_t a,u32_t b,u32_t c,u32_t d); 
extern void ip_addr_set(struct ip_addr* dest,struct ip_addr* src);
extern u8_t ip_addr_maskcmp(struct ip_addr* addr1,struct ip_addr*  addr2,struct ip_addr* mask);
extern u8_t ip_addr_cmp(struct ip_addr* addr1,struct ip_addr* addr2);
extern u8_t ip_addr_isany(struct ip_addr* addr1);
extern u8_t ip_addr_isbroadcast(struct ip_addr* addr1,struct ip_addr*  mask) ;
extern u8_t ip_addr_ismulticast(struct ip_addr* addr1);
//extern void ip_addr_debug_print(debug, ipaddr) ;
/* cast to unsigned int, as it is used as argument to printf functions
 * which expect integer arguments */
extern unsigned int  ip4_addr1(struct ip_addr* ipaddr);
extern unsigned int  ip4_addr2(struct ip_addr* ipaddr);
extern unsigned int  ip4_addr3(struct ip_addr* ipaddr);
extern unsigned int  ip4_addr4(struct ip_addr* ipaddr);
//-------------------------------------------------------------------------------------

#endif

#ifdef Eth_IP_FRAG
#include "lwip/pbuf.h"
#include "lwip/ip_addr.h"
//-------------------------------------------------------------------------------------

extern struct pbuf * ip_reass(struct pbuf *p);
extern err_t ip_frag(struct pbuf *p, struct netif *netif, struct ip_addr *dest);
//-------------------------------------------------------------------------------------

#endif

#ifdef Eth_SNMP
//-------------------------------------------------------------------------------------
#include "lwip/opt.h"
//Estas funciones estn definidas pero no implementadas
//pero se les llama desde otras capas si en su momento se implementan para 
//q todo funcione bien

/* network interface */
extern void snmp_add_ifinoctets(unsigned long value);
extern void snmp_inc_ifinucastpkts(void);
extern void snmp_inc_ifinnucastpkts(void);
extern void snmp_inc_ifindiscards(void);
extern void snmp_add_ifoutoctets(unsigned long value);
extern void snmp_inc_ifoutucastpkts(void);
extern void snmp_inc_ifoutnucastpkts(void);
extern void snmp_inc_ifoutdiscards(void);

/* IP */
extern void snmp_inc_ipinreceives(void);
extern void snmp_inc_ipindelivers(void);
extern void snmp_inc_ipindiscards(void);
extern void snmp_inc_ipoutdiscards(void);
extern void snmp_inc_ipoutrequests(void);
extern void snmp_inc_ipunknownprotos(void);
extern void snmp_inc_ipnoroutes(void);
extern void snmp_inc_ipforwdatagrams(void);

/* ICMP */
extern void snmp_inc_icmpinmsgs(void);
extern void snmp_inc_icmpinerrors(void);
extern void snmp_inc_icmpindestunreachs(void);
extern void snmp_inc_icmpintimeexcds(void);
extern void snmp_inc_icmpinparmprobs(void);
extern void snmp_inc_icmpinsrcquenchs(void);
extern void snmp_inc_icmpinredirects(void);
extern void snmp_inc_icmpinechos(void);
extern void snmp_inc_icmpinechoreps(void);
extern void snmp_inc_icmpintimestamps(void);
extern void snmp_inc_icmpintimestampreps(void);
extern void snmp_inc_icmpinaddrmasks(void);
extern void snmp_inc_icmpinaddrmaskreps(void);
extern void snmp_inc_icmpoutmsgs(void);
extern void snmp_inc_icmpouterrors(void);
extern void snmp_inc_icmpoutdestunreachs(void);
extern void snmp_inc_icmpouttimeexcds(void);
extern void snmp_inc_icmpoutparmprobs(void);
extern void snmp_inc_icmpoutsrcquenchs(void);
extern void snmp_inc_icmpoutredirects(void); 
extern void snmp_inc_icmpoutechos(void);
extern void snmp_inc_icmpoutechoreps(void);
extern void snmp_inc_icmpouttimestamps(void);
extern void snmp_inc_icmpouttimestampreps(void);
extern void snmp_inc_icmpoutaddrmasks(void);
extern void snmp_inc_icmpoutaddrmaskreps(void);

/* TCP */
extern void snmp_inc_tcpactiveopens(void);
extern void snmp_inc_tcppassiveopens(void);
extern void snmp_inc_tcpattemptfails(void);
extern void snmp_inc_tcpestabresets(void);
extern void snmp_inc_tcpcurrestab(void);
extern void snmp_inc_tcpinsegs(void);
extern void snmp_inc_tcpoutsegs(void);
extern void snmp_inc_tcpretranssegs(void);
extern void snmp_inc_tcpinerrs(void);
extern void snmp_inc_tcpoutrsts(void);

/* UDP */
extern void snmp_inc_udpindatagrams(void);
extern void snmp_inc_udpnoports(void);
extern void snmp_inc_udpinerrors(void);
extern void snmp_inc_udpoutdatagrams(void);

//-------------------------------------------------------------------------------------

#endif


#endif